@@ -19,63 +19,76 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
-
 #include <libxfce4util/libxfce4util.h>
-#include <libxfce4ui/libxfce4ui.h>
 
-#include <libxfce4kbd-private/xfce-shortcuts.h>
-#include <libxfce4kbd-private/xfce-shortcut-dialog.h>
-#include <libxfce4kbd-private/xfce-shortcuts-editor.h>
+#include "xfce-shortcut-dialog.h"
+#include "xfce-shortcuts-editor.h"
+#include "xfce-shortcuts.h"
 
 
 
 typedef struct
 {
   XfceShortcutsEditor *editor;
-  XfceGtkActionEntry  *entry;
-  const gchar         *displayed_label;
+  XfceGtkActionEntry *entry;
+  size_t section_index;
+  const gchar *displayed_label;
 } ShortcutEditClickedData;
 
 typedef struct
 {
-  GtkWidget           *shortcut_button;
-  XfceGtkActionEntry  *entry;
+  XfceShortcutsEditor *editor;
+  GtkWidget *shortcut_button;
+  XfceGtkActionEntry *entry;
+  size_t section_index;
 } ShortcutOtherClickedData;
 
 typedef struct
 {
-  gboolean        in_use;
+  gboolean in_use;
   GdkModifierType mods;
-  guint           key;
-  const gchar    *current_path;
-  gchar          *other_path;
+  guint key;
+  const gchar *current_path;
+  gchar *other_path;
+
+  /* A set of all accel paths that are allowed to have the same key+modifiers as 'current_path'
+   * If no overlap is allowed, this will be NULL. */
+  GHashTable *paths_with_overlap_allowed; /* gchar *accel_path (no values, used as set) */
+
 } ShortcutInfo;
 
 
 
-static void     xfce_shortcuts_editor_finalize                 (GObject                   *object);
-static void     xfce_shortcuts_editor_create_contents          (XfceShortcutsEditor       *editor);
-static void     xfce_shortcuts_editor_shortcut_clicked         (GtkWidget                 *widget,
-                                                                ShortcutEditClickedData   *data);
-static void     xfce_shortcuts_editor_shortcut_clear_clicked   (GtkWidget                 *widget,
-                                                                ShortcutOtherClickedData  *data);
-static void     xfce_shortcuts_editor_shortcut_reset_clicked   (GtkWidget                 *widget,
-                                                                ShortcutOtherClickedData  *data);
-static void     xfce_shortcuts_editor_shortcut_check           (gpointer                   data,
-                                                                const gchar               *path,
-                                                                guint                      key,
-                                                                GdkModifierType            mods,
-                                                                gboolean                   changed);
-static gboolean xfce_shortcuts_editor_validate_shortcut        (XfceShortcutDialog        *editor,
-                                                                const gchar               *shortcut,
-                                                                ShortcutEditClickedData   *data);
-static void     free_data                                      (gpointer                   data,
-                                                                GClosure                  *closure);
+static void
+xfce_shortcuts_editor_finalize (GObject *object);
+static void
+xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor);
+static void
+xfce_shortcuts_editor_shortcut_clicked (GtkWidget *widget,
+                                        ShortcutEditClickedData *data);
+static void
+xfce_shortcuts_editor_shortcut_clear_clicked (GtkWidget *widget,
+                                              ShortcutOtherClickedData *data);
+static void
+xfce_shortcuts_editor_shortcut_reset_clicked (GtkWidget *widget,
+                                              ShortcutOtherClickedData *data);
+static void
+xfce_shortcuts_editor_shortcut_check (gpointer data,
+                                      const gchar *path,
+                                      guint key,
+                                      GdkModifierType mods,
+                                      gboolean changed);
+static gboolean
+xfce_shortcuts_editor_validate_shortcut (XfceShortcutDialog *editor,
+                                         const gchar *shortcut,
+                                         ShortcutEditClickedData *data);
+static void
+free_data (gpointer data,
+           GClosure *closure);
 
 
 
@@ -86,10 +99,16 @@ struct _XfceShortcutsEditorClass
 
 struct _XfceShortcutsEditor
 {
-  GtkVBox             __parent__;
+  GtkVBox __parent__;
 
   XfceShortcutsEditorSection *arrays;
-  size_t                      arrays_count;
+  size_t arrays_count;
+
+  /* Each group in this list is an array of XfceShortcutsEditorSection indexes
+   * (into 'arrays' above). Within the sections listed in a group, shortcuts
+   * are allowed to overlap (key+modifiers can be the same).
+   * A section can be in multiple groups.*/
+  GList *groups_with_overlap_allowed; /* GArray<size_t arrays_index> */
 };
 
 
@@ -128,6 +147,12 @@ xfce_shortcuts_editor_finalize (GObject *object)
 {
   XfceShortcutsEditor *editor = XFCE_SHORTCUTS_EDITOR (object);
 
+  for (GList *l = editor->groups_with_overlap_allowed; l != NULL; l = l->next)
+    {
+      g_array_free (l->data, TRUE);
+    }
+  g_list_free (editor->groups_with_overlap_allowed);
+
   for (size_t i = 0; i < editor->arrays_count; i++)
     g_free (editor->arrays[i].section_name);
   g_free (editor->arrays);
@@ -145,11 +170,11 @@ xfce_shortcuts_editor_finalize (GObject *object)
  *
  * Since: 4.17.2
  **/
-GtkWidget*
+GtkWidget *
 xfce_shortcuts_editor_new (int argument_count, ...)
 {
   GtkWidget *editor;
-  va_list    argument_list;
+  va_list argument_list;
 
   va_start (argument_list, argument_count);
 
@@ -172,9 +197,9 @@ xfce_shortcuts_editor_new (int argument_count, ...)
  *
  * Since: 4.17.5
  **/
-GtkWidget*
+GtkWidget *
 xfce_shortcuts_editor_new_array (XfceShortcutsEditorSection *sections,
-                                 int                         n_sections)
+                                 int n_sections)
 {
   XfceShortcutsEditor *editor;
 
@@ -217,8 +242,8 @@ xfce_shortcuts_editor_new_array (XfceShortcutsEditorSection *sections,
  *
  * Since: 4.17.2
  **/
-GtkWidget*
-xfce_shortcuts_editor_new_variadic (int     argument_count,
+GtkWidget *
+xfce_shortcuts_editor_new_variadic (int argument_count,
                                     va_list argument_list)
 {
   XfceShortcutsEditor *editor;
@@ -233,8 +258,8 @@ xfce_shortcuts_editor_new_variadic (int     argument_count,
 
   for (int i = 0; i * 3 + 1 < argument_count; i++)
     {
-      editor->arrays[i].section_name = g_strdup (va_arg (argument_list, gchar*));
-      editor->arrays[i].entries = va_arg (argument_list, XfceGtkActionEntry*);
+      editor->arrays[i].section_name = g_strdup (va_arg (argument_list, gchar *));
+      editor->arrays[i].entries = va_arg (argument_list, XfceGtkActionEntry *);
       editor->arrays[i].size = va_arg (argument_list, size_t);
     }
 
@@ -247,8 +272,97 @@ xfce_shortcuts_editor_new_variadic (int     argument_count,
 
 
 
+/**
+ * xfce_shortcuts_editor_add_overlap_group:
+ * @editor: A #XfceShortcutsEditor.
+ * @first_section_index: The first index in the array of #XfceShortcutsEditorSection.
+ * @...: Other indexes that should be in the group, as ints, terminated with -1.
+ *
+ * Adds an overlap group to @editor, which consists of a list of
+ * #XfceShortcutsEditorSection indexes. Within actions inside all sections of
+ * the group, there may be duplicate shortcuts set.
+ *
+ * Don't forget the terminating -1 at the end of the variable list of indexes.
+ **/
+void
+xfce_shortcuts_editor_add_overlap_group (XfceShortcutsEditor *editor,
+                                         int first_section_index,
+                                         ...)
+{
+  /* NB: This function takes ints instead of ssize_t because the compiler, at
+   * the call site, will not know that the varargs are supposed to be ssize_t,
+   * but will assume they are just ints.  Since those types are not the same
+   * size, we'll end up with invalid data in the function.  Expecting the
+   * caller to know this and cast to ssize_t is unreasonable.  But when we put
+   * the values into the GArray, they need to be the correct type and size for
+   * the array. */
+
+  g_return_if_fail (XFCE_IS_SHORTCUTS_EDITOR (editor));
+  g_return_if_fail (first_section_index >= 0);
+
+  GArray *group_indexes = g_array_sized_new (FALSE, TRUE, sizeof (size_t), 2);
+  size_t first_section_index_pos = first_section_index;
+  g_array_append_val (group_indexes, first_section_index_pos);
+
+  va_list argument_list;
+  va_start (argument_list, first_section_index);
+
+  for (int index = va_arg (argument_list, int); index >= 0; index = va_arg (argument_list, int))
+    {
+      size_t index_pos = index;
+#ifndef G_DISABLE_CHECKS
+      if (index_pos >= editor->arrays_count)
+        {
+          va_end (argument_list);
+          g_return_if_fail (index_pos < editor->arrays_count);
+          return; // Shut up gcc-analyzer
+        }
+#endif
+      g_array_append_val (group_indexes, index_pos);
+    }
+
+  va_end (argument_list);
+
+  editor->groups_with_overlap_allowed = g_list_prepend (editor->groups_with_overlap_allowed, group_indexes);
+}
+
+
+
+/**
+ * xfce_shortcuts_editor_add_overlap_group_array:
+ * @editor: A #XfceShortcutsEditor.
+ * @section_indexes: An array of indexes.
+ * @n_section_indexes: The count of elements in @section_indexes.
+ *
+ * Adds an overlap group to @editor, which consists of a list of
+ * #XfceShortcutsEditorSection indexes.  Within actions inside all sections of
+ * the group, there may be duplicate shortcuts set.
+ **/
+void
+xfce_shortcuts_editor_add_overlap_group_array (XfceShortcutsEditor *editor,
+                                               size_t *section_indexes,
+                                               size_t n_section_indexes)
+{
+  g_return_if_fail (XFCE_IS_SHORTCUTS_EDITOR (editor));
+  g_return_if_fail (section_indexes != NULL);
+  g_return_if_fail (n_section_indexes > 0);
+#ifndef G_DISABLE_CHECKS
+  for (size_t i = 0; i < n_section_indexes; ++i)
+    {
+      g_return_if_fail (section_indexes[i] < editor->arrays_count);
+    }
+#endif
+
+  GArray *group_indexes = g_array_sized_new (FALSE, FALSE, sizeof (size_t), n_section_indexes);
+  memcpy (group_indexes->data, section_indexes, sizeof (*section_indexes) * n_section_indexes);
+  group_indexes->len = n_section_indexes;
+  editor->groups_with_overlap_allowed = g_list_prepend (editor->groups_with_overlap_allowed, group_indexes);
+}
+
+
+
 static void
-free_data (gpointer  data,
+free_data (gpointer data,
            GClosure *closure)
 {
   g_free (data);
@@ -260,17 +374,17 @@ static void
 xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor)
 {
   GtkAdjustment *adjustment;
-  GtkWidget     *vbox;
-  GtkWidget     *frame;
-  GtkWidget     *swin;
-  GtkWidget     *grid;
-  GtkWidget     *label;
-  GtkWidget     *box;
-  GtkWidget     *box2;
-  GtkWidget     *shortcut_button;
-  GtkWidget     *clear_button;
-  GtkWidget     *reset_button;
-  size_t         row;
+  GtkWidget *vbox;
+  GtkWidget *frame;
+  GtkWidget *swin;
+  GtkWidget *grid;
+  GtkWidget *label;
+  GtkWidget *box;
+  GtkWidget *box2;
+  GtkWidget *shortcut_button;
+  GtkWidget *clear_button;
+  GtkWidget *reset_button;
+  size_t row;
 
   vbox = GTK_WIDGET (editor);
 
@@ -313,9 +427,9 @@ xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor)
         }
 
       /* section name */
-      if (! xfce_str_is_empty (editor->arrays[array_idx].section_name))
+      if (!xfce_str_is_empty (editor->arrays[array_idx].section_name))
         {
-          label  = gtk_label_new ("");
+          label = gtk_label_new ("");
           markup = g_strconcat ("<b>", editor->arrays[array_idx].section_name, "</b>", NULL);
           gtk_label_set_markup (GTK_LABEL (label), markup);
           g_free (markup);
@@ -330,12 +444,12 @@ xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor)
       /* section shortcut entries */
       for (size_t entry_idx = 0; entry_idx < editor->arrays[array_idx].size; entry_idx++)
         {
-          ShortcutEditClickedData  *data;
+          ShortcutEditClickedData *data;
           ShortcutOtherClickedData *clear_data;
           ShortcutOtherClickedData *reset_data;
-          XfceGtkActionEntry       entry;
-          GtkAccelKey               key;
-          gchar                    *shortcut_text;
+          XfceGtkActionEntry entry;
+          GtkAccelKey key;
+          gchar *shortcut_text;
 
           entry = editor->arrays[array_idx].entries[entry_idx];
 
@@ -382,6 +496,7 @@ xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor)
 
           data->editor = editor;
           data->entry = editor->arrays[array_idx].entries + entry_idx;
+          data->section_index = array_idx;
           g_signal_connect_data (G_OBJECT (shortcut_button), "clicked", G_CALLBACK (xfce_shortcuts_editor_shortcut_clicked), data, free_data, 0);
 
           /* clear button */
@@ -390,8 +505,10 @@ xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor)
           gtk_box_pack_end (GTK_BOX (box2), clear_button, FALSE, TRUE, 0);
           gtk_widget_show (clear_button);
 
+          clear_data->editor = editor;
           clear_data->shortcut_button = shortcut_button;
           clear_data->entry = editor->arrays[array_idx].entries + entry_idx;
+          clear_data->section_index = array_idx;
           g_signal_connect_data (G_OBJECT (clear_button), "clicked", G_CALLBACK (xfce_shortcuts_editor_shortcut_clear_clicked), clear_data, free_data, 0);
 
           /* reset button */
@@ -400,8 +517,10 @@ xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor)
           gtk_box_pack_end (GTK_BOX (box), reset_button, FALSE, FALSE, 0);
           gtk_widget_show (reset_button);
 
+          reset_data->editor = editor;
           reset_data->shortcut_button = shortcut_button;
           reset_data->entry = editor->arrays[array_idx].entries + entry_idx;
+          reset_data->section_index = array_idx;
           g_signal_connect_data (G_OBJECT (reset_button), "clicked", G_CALLBACK (xfce_shortcuts_editor_shortcut_reset_clicked), reset_data, free_data, 0);
 
           row++;
@@ -411,20 +530,33 @@ xfce_shortcuts_editor_create_contents (XfceShortcutsEditor *editor)
 
 
 
-static void
-xfce_shortcuts_editor_shortcut_check (gpointer        data,
-                                      const gchar    *path,
-                                      guint           key,
-                                      GdkModifierType mods,
-                                      gboolean        changed)
+static inline gboolean
+shortcut_in_overlap_group (ShortcutInfo *info,
+                           const gchar *path)
 {
-  ShortcutInfo *info = (ShortcutInfo*) data;
+  if (info->paths_with_overlap_allowed == NULL)
+    return FALSE;
+
+  return g_hash_table_contains (info->paths_with_overlap_allowed, path);
+}
+
+
+
+static void
+xfce_shortcuts_editor_shortcut_check (gpointer data,
+                                      const gchar *path,
+                                      guint key,
+                                      GdkModifierType mods,
+                                      gboolean changed)
+{
+  ShortcutInfo *info = (ShortcutInfo *) data;
   if (info->in_use)
     return;
 
-  info->in_use = info->mods == mods &&
-                 info->key == key &&
-                 g_strcmp0 (info->current_path, path) != 0;
+  info->in_use = info->mods == mods
+                 && info->key == key
+                 && g_strcmp0 (info->current_path, path) != 0
+                 && !shortcut_in_overlap_group (info, path);
 
   if (info->in_use)
     info->other_path = g_strdup (path);
@@ -432,15 +564,81 @@ xfce_shortcuts_editor_shortcut_check (gpointer        data,
 
 
 
+static GHashTable *
+build_paths_with_overlap_allowed_set (XfceShortcutsEditor *editor,
+                                      size_t changed_section_index)
+{
+  if (editor->groups_with_overlap_allowed != NULL)
+    {
+      GHashTable *paths_with_overlap_allowed = NULL;
+      GHashTable *sections_with_overlap_allowed = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+      /* First build a list of section indexes that this accelerator's section can overlap with */
+      for (GList *l = editor->groups_with_overlap_allowed; l != NULL; l = l->next)
+        {
+          GArray *overlap_allowed_group = l->data;
+
+          gboolean overlap_allowed = FALSE;
+          for (gsize group_idx = 0; group_idx < overlap_allowed_group->len; ++group_idx)
+            {
+              size_t section_index = g_array_index (overlap_allowed_group, size_t, group_idx);
+              if (section_index == changed_section_index)
+                {
+                  overlap_allowed = TRUE;
+                  break;
+                }
+            }
+
+          if (overlap_allowed)
+            {
+              for (gsize group_idx = 0; group_idx < overlap_allowed_group->len; ++group_idx)
+                {
+                  size_t section_index = g_array_index (overlap_allowed_group, size_t, group_idx);
+
+                  /* Dont allow overlap within our own section */
+                  if (changed_section_index != section_index)
+                    g_hash_table_add (sections_with_overlap_allowed, GUINT_TO_POINTER (section_index));
+                }
+            }
+        }
+
+      paths_with_overlap_allowed = g_hash_table_new (g_str_hash, g_str_equal);
+
+      /* Insert paths that are allowed to overlap with this accelerator into 'paths_with_overlap_allowed' */
+      for (size_t array_idx = 0; array_idx < editor->arrays_count; ++array_idx)
+        {
+          XfceShortcutsEditorSection *section = &editor->arrays[array_idx];
+          for (size_t entry_idx = 0; entry_idx < section->size; ++entry_idx)
+            {
+              gchar *accel_path = (gchar *) section->entries[entry_idx].accel_path;
+              if (accel_path != NULL && g_hash_table_contains (sections_with_overlap_allowed, GUINT_TO_POINTER (array_idx)))
+                {
+                  g_hash_table_add (paths_with_overlap_allowed, accel_path);
+                }
+            }
+        }
+
+      g_hash_table_destroy (sections_with_overlap_allowed);
+
+      return paths_with_overlap_allowed;
+    }
+  else
+    {
+      return NULL;
+    }
+}
+
+
+
 static gboolean
-xfce_shortcuts_editor_validate_shortcut (XfceShortcutDialog      *editor,
-                                         const gchar             *shortcut,
+xfce_shortcuts_editor_validate_shortcut (XfceShortcutDialog *editor,
+                                         const gchar *shortcut,
                                          ShortcutEditClickedData *data)
 {
   GdkModifierType accel_mods;
-  guint           accel_key;
-  ShortcutInfo    info;
-  gchar          *command, *message;
+  guint accel_key;
+  ShortcutInfo info;
+  gchar *command, *message;
 
   g_return_val_if_fail (XFCE_IS_SHORTCUT_DIALOG (editor), FALSE);
   g_return_val_if_fail (shortcut != NULL, FALSE);
@@ -458,15 +656,16 @@ xfce_shortcuts_editor_validate_shortcut (XfceShortcutDialog      *editor,
   info.key = accel_key;
   info.current_path = data->entry->accel_path;
   info.other_path = NULL;
+  info.paths_with_overlap_allowed = build_paths_with_overlap_allowed_set (data->editor, data->section_index);
 
   gtk_accel_map_foreach_unfiltered (&info, xfce_shortcuts_editor_shortcut_check);
 
+  g_clear_pointer (&info.paths_with_overlap_allowed, g_hash_table_destroy);
+
   if (info.in_use)
     {
       command = g_strrstr (info.other_path, "/");
-      command = command == NULL ?
-                info.other_path :
-                command + 1; /* skip leading slash */
+      command = command == NULL ? info.other_path : command + 1; /* skip leading slash */
 
       message = g_strdup_printf (_("This keyboard shortcut is currently used by: '%s'"),
                                  command);
@@ -477,7 +676,7 @@ xfce_shortcuts_editor_validate_shortcut (XfceShortcutDialog      *editor,
 
       return FALSE;
     }
-  else if (! gtk_accel_map_change_entry (data->entry->accel_path, accel_key, accel_mods, TRUE))
+  else if (!gtk_accel_map_change_entry (data->entry->accel_path, accel_key, accel_mods, TRUE))
     return FALSE;
 
   return TRUE;
@@ -486,15 +685,15 @@ xfce_shortcuts_editor_validate_shortcut (XfceShortcutDialog      *editor,
 
 
 static void
-xfce_shortcuts_editor_shortcut_clicked (GtkWidget               *widget,
+xfce_shortcuts_editor_shortcut_clicked (GtkWidget *widget,
                                         ShortcutEditClickedData *data)
 {
-  GtkWidget       *dialog;
-  gint             response;
-  const gchar     *shortcut;
-  GdkModifierType  accel_mods;
-  guint            accel_key;
-  gchar           *label;
+  GtkWidget *dialog;
+  gint response;
+  const gchar *shortcut;
+  GdkModifierType accel_mods;
+  guint accel_key;
+  gchar *label;
 
   dialog = xfce_shortcut_dialog_new ("", data->displayed_label, "");
 
@@ -519,8 +718,8 @@ xfce_shortcuts_editor_shortcut_clicked (GtkWidget               *widget,
 
 
 static void
-xfce_shortcuts_editor_shortcut_clear_clicked  (GtkWidget                 *widget,
-                                               ShortcutOtherClickedData  *data)
+xfce_shortcuts_editor_shortcut_clear_clicked (GtkWidget *widget,
+                                              ShortcutOtherClickedData *data)
 {
   if (gtk_accel_map_change_entry (data->entry->accel_path, 0, 0, TRUE))
     gtk_button_set_label (GTK_BUTTON (data->shortcut_button), "...");
@@ -529,14 +728,14 @@ xfce_shortcuts_editor_shortcut_clear_clicked  (GtkWidget                 *widget
 
 
 static void
-xfce_shortcuts_editor_shortcut_reset_clicked  (GtkWidget                 *widget,
-                                               ShortcutOtherClickedData  *data)
+xfce_shortcuts_editor_shortcut_reset_clicked (GtkWidget *widget,
+                                              ShortcutOtherClickedData *data)
 {
-  GdkModifierType  accel_mods;
-  guint            accel_key;
-  gchar           *label;
-  ShortcutInfo     info;
-  gchar           *command, *message;
+  GdkModifierType accel_mods;
+  guint accel_key;
+  gchar *label;
+  ShortcutInfo info;
+  gchar *command, *message;
 
   gtk_accelerator_parse (data->entry->default_accelerator, &accel_key, &accel_mods);
 
@@ -546,16 +745,17 @@ xfce_shortcuts_editor_shortcut_reset_clicked  (GtkWidget                 *widget
   info.key = accel_key;
   info.current_path = data->entry->accel_path;
   info.other_path = NULL;
+  info.paths_with_overlap_allowed = build_paths_with_overlap_allowed_set (data->editor, data->section_index);
 
   gtk_accel_map_foreach_unfiltered (&info, xfce_shortcuts_editor_shortcut_check);
 
+  g_clear_pointer (&info.paths_with_overlap_allowed, g_hash_table_destroy);
+
   /* an empty default accelerator is always available */
   if (g_strcmp0 (data->entry->default_accelerator, "") != 0 && info.in_use == TRUE)
     {
       command = g_strrstr (info.other_path, "/");
-      command = command == NULL ?
-                info.other_path :
-                command + 1; /* skip leading slash */
+      command = command == NULL ? info.other_path : command + 1; /* skip leading slash */
 
       message = g_strdup_printf (_("This keyboard shortcut is currently used by: '%s'"),
                                  command);
@@ -576,4 +776,4 @@ xfce_shortcuts_editor_shortcut_reset_clicked  (GtkWidget                 *widget
     }
 
   g_free (info.other_path);
-}
\ No newline at end of file
+}
