@@ -85,6 +85,8 @@ xfce_about_system (GtkBuilder *builder)
   const gchar *windowing_system_text = "";
   guint num_gpus = 0;
 
+  glibtop_init ();
+
   os_logo = get_os_logo ();
   if (os_logo)
     {
@@ -320,14 +322,7 @@ xfce_about_credits (GtkTextBuffer *buffer)
           for (user = group->contributors; user->name != NULL; user++)
             {
               gtk_text_buffer_insert_with_tags (buffer, &end, user->name, -1, indent, NULL);
-              if (g_strcmp0 (user->email, "") != 0)
-                {
-                  gtk_text_buffer_insert (buffer, &end, " <", -1);
-                  gtk_text_buffer_insert (buffer, &end, user->email, -1);
-                  gtk_text_buffer_insert (buffer, &end, ">\n", -1);
-                }
-              else
-                gtk_text_buffer_insert (buffer, &end, "\n", -1);
+              gtk_text_buffer_insert (buffer, &end, "\n", -1);
             }
         }
       else
